# Generated by Django 5.1.1 on 2024-11-22 16:32

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_tasks_database", "0011_rename_complete_status"),
    ]

    operations = [
        migrations.AddField(
            model_name="dbtaskresult",
            name="exception_class_path",
            field=models.TextField(default="", verbose_name="exception class path"),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="dbtaskresult",
            name="traceback",
            field=models.TextField(default="", verbose_name="traceback"),
            preserve_default=False,
        ),
    ]
