# Generated by Django 5.1.1 on 2024-11-22 16:32

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("django_tasks_database", "0013_separate_exception_fields"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="dbtaskresult",
            name="exception_data",
        ),
    ]
