# Generated by Django 5.0.6 on 2024-06-13 10:24

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_tasks_database", "0002_alter_dbtaskresult_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="dbtaskresult",
            name="enqueued_at",
            field=models.DateTimeField(
                auto_now_add=True, default=django.utils.timezone.now
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="dbtaskresult",
            name="finished_at",
            field=models.DateTimeField(null=True),
        ),
    ]
