# Generated by Django 4.2.13 on 2024-11-21 13:46

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_tasks_database", "0009_remove_results_field"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dbtaskresult",
            name="status",
            field=models.CharField(
                choices=[
                    ("NEW", "New"),
                    ("RUNNING", "Running"),
                    ("FAILED", "Failed"),
                    ("SUCCEEDED", "Succeeded"),
                ],
                default="NEW",
                max_length=9,
                verbose_name="status",
            ),
        ),
    ]
