# Generated by Django 4.2.13 on 2024-06-08 15:24

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_tasks_database", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="dbtaskresult",
            options={
                "ordering": [
                    models.OrderBy(models.F("priority"), descending=True),
                    models.OrderBy(
                        models.F("run_after"), descending=True, nulls_last=True
                    ),
                ],
                "verbose_name": "Task Result",
                "verbose_name_plural": "Task Results",
            },
        ),
    ]
