# Generated by Django 4.2.13 on 2024-08-23 14:51

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("django_tasks_database", "0008_separate_results_field"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="dbtaskresult",
            name="result",
        ),
    ]
