# Generated by Django 5.1.5 on 2025-04-27 12:00

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_tasks_database", "0014_remove_dbtaskresult_exception_data"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="dbtaskresult",
            options={
                "ordering": [
                    models.OrderBy(models.F("priority"), descending=True),
                    models.OrderBy(models.F("run_after"), nulls_last=True),
                ],
                "verbose_name": "Task Result",
                "verbose_name_plural": "Task Results",
            },
        ),
    ]
