# Generated by Django 4.2.13 on 2024-08-23 14:38

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_tasks_database", "0006_alter_dbtaskresult_args_kwargs_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="dbtaskresult",
            name="exception_data",
            field=models.JSONField(
                default=None, null=True, verbose_name="exception data"
            ),
        ),
        migrations.AddField(
            model_name="dbtaskresult",
            name="return_value",
            field=models.JSONField(
                default=None, null=True, verbose_name="return value"
            ),
        ),
    ]
