# Generated by Django 4.2.14 on 2024-08-02 13:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_tasks_database", "0005_alter_dbtaskresult_priority_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dbtaskresult",
            name="args_kwargs",
            field=models.JSONField(verbose_name="args kwargs"),
        ),
        migrations.AlterField(
            model_name="dbtaskresult",
            name="backend_name",
            field=models.TextField(verbose_name="backend name"),
        ),
        migrations.AlterField(
            model_name="dbtaskresult",
            name="enqueued_at",
            field=models.DateTimeField(auto_now_add=True, verbose_name="enqueued at"),
        ),
        migrations.AlterField(
            model_name="dbtaskresult",
            name="finished_at",
            field=models.DateTimeField(null=True, verbose_name="finished at"),
        ),
        migrations.AlterField(
            model_name="dbtaskresult",
            name="priority",
            field=models.IntegerField(default=0, verbose_name="priority"),
        ),
        migrations.AlterField(
            model_name="dbtaskresult",
            name="queue_name",
            field=models.TextField(default="default", verbose_name="queue name"),
        ),
        migrations.AlterField(
            model_name="dbtaskresult",
            name="result",
            field=models.JSONField(default=None, null=True, verbose_name="result"),
        ),
        migrations.AlterField(
            model_name="dbtaskresult",
            name="run_after",
            field=models.DateTimeField(null=True, verbose_name="run after"),
        ),
        migrations.AlterField(
            model_name="dbtaskresult",
            name="started_at",
            field=models.DateTimeField(null=True, verbose_name="started at"),
        ),
        migrations.AlterField(
            model_name="dbtaskresult",
            name="status",
            field=models.CharField(
                choices=[
                    ("NEW", "New"),
                    ("RUNNING", "Running"),
                    ("FAILED", "Failed"),
                    ("COMPLETE", "Complete"),
                ],
                default="NEW",
                max_length=8,
                verbose_name="status",
            ),
        ),
        migrations.AlterField(
            model_name="dbtaskresult",
            name="task_path",
            field=models.TextField(verbose_name="task path"),
        ),
    ]
